﻿using UnityEngine;
using System.Linq;
using System.Collections;

public class Movement : MonoBehaviour
{
    public Vector3 forward;
    public Vector2 movementDirection;
    public float rotationAngle;
    public float speed = 20;
    public float Vspeed = 0;
    public float minVSpeed = -15f;
    public float gravity = 1f;
    public bool followGravity = true;

    private Transform transf;
    private Transform childTransf;
    public float effectiveRadius;
    private bool landed = false;
    public float radius = 0.6f;
    private float angle = 0.0f;
    private Quaternion rotation = Quaternion.identity;
    private Vector3 direction = Vector3.one;

    // Use this for initialization
    void Start()
    {
        //effectiveRadius = 0;
        transf = GetComponent<Transform>();
        childTransf = GetComponentsInChildren<MovedObject>().Select(x => x.GetComponent<Transform>()).First();
        direction = new Vector3(Mathf.Sin(angle), Mathf.Cos(angle));
    }

    // Update is called once per frame
    void Update()
    {
        if (movementDirection.x != 0 || movementDirection.y != 0)
            Translate(movementDirection.x * speed * Time.deltaTime, movementDirection.y * speed * Time.deltaTime);
        if (followGravity)
        {
            Vspeed = Mathf.Max(minVSpeed, Vspeed - gravity * Time.deltaTime);
            RaycastHit hit;
            Vector3 maxDist = rotation * Vector3.forward * radius;
            var layerMask = 1 << 8;
            if (Physics.Raycast(maxDist, -maxDist, out hit, Mathf.Infinity, layerMask))
            {
                var expectedRadius = radius - hit.distance + 0.8f;
                effectiveRadius += Vspeed * Time.deltaTime;
                if (expectedRadius >= effectiveRadius)
                {
                    effectiveRadius = expectedRadius;
                    landed = true;
                }
                else
                    landed = false;
            }
        }

        UpdatePositionRotation();
        childTransf.localRotation = Quaternion.Euler(0, rotationAngle * Mathf.Rad2Deg, 0);
    }

    public void Translate(float x, float y)
    {
        var perpendicular = new Vector3(-direction.y, direction.x);
        var verticalRotation = Quaternion.AngleAxis(y, perpendicular);
        var horizontalRotation = Quaternion.AngleAxis(x, direction);
        rotation *= horizontalRotation * verticalRotation;
    }

    void UpdatePositionRotation()
    {
        transform.localPosition = rotation * Vector3.forward * effectiveRadius;
        transform.rotation = rotation * Quaternion.LookRotation(direction, Vector3.forward);
    }

    void OnDrawGizmos()
    {
        Vector3 maxDist = rotation * Vector3.forward * radius;
        Vector3 center = Vector3.zero;
        Gizmos.DrawRay(maxDist, -maxDist);
    }

    public void Copy(Movement source)
    {
        this.direction = source.direction;
        this.rotation = source.rotation;
        this.effectiveRadius = source.effectiveRadius;
        UpdatePositionRotation();
    }
}