﻿using UnityEngine;
using System.Collections;

public class PlayerKeyboardController : MonoBehaviour
{
    Movement movement = null;
    ActionManager actionManager = null;
    // Use this for initialization
    void Start()
    {
        movement = GetComponent<Movement>();
        actionManager = GetComponent<ActionManager>();
    }

    // Update is called once per frame
    void Update()
    {
        ManageKeyboardMovementInput();
        MouseRotationInput();
        MouseButtonClick();
    }

    private void MouseButtonClick()
    {
        if(Input.GetButton("Fire1"))
        {
            actionManager.shoot = true;
        }
        else
        {
            actionManager.shoot = false;
        }
    }

    private void MouseRotationInput()
    {
        var screenDimensions = new Vector3(Screen.width, Screen.height, 0);
        var mousePosition = Input.mousePosition - screenDimensions / 2;
        var relativePosition = new Vector2(mousePosition.x / screenDimensions.x, mousePosition.y / screenDimensions.y);
        //Debug.Log((Mathf.Atan2(relativePosition.x, relativePosition.y) + Mathf.PI) * Mathf.Rad2Deg);
        movement.rotationAngle = Mathf.Atan2(relativePosition.x, relativePosition.y) + Mathf.PI;
    }

    private void ManageKeyboardMovementInput()
    {
        var horizontalDirection = -Input.GetAxis("Horizontal");
        var verticalDirection = Input.GetAxis("Vertical");

        if (Input.GetKey(KeyCode.A))
        {
            horizontalDirection += 1;
        }
        if (Input.GetKey(KeyCode.D))
        {
            horizontalDirection += -1;
        }
        if (Input.GetKey(KeyCode.W))
        {
            verticalDirection += 1;
        }
        if (Input.GetKey(KeyCode.S))
        {
            verticalDirection += -1;
        }


        if (verticalDirection == 0 && horizontalDirection == 0)
        {
            movement.movementDirection = Vector2.zero;
        }
        else
        {
            var angle = 0;
            if (verticalDirection == 0)
            {
                angle = horizontalDirection > 0 ? 0 : 180;
            }
            else if (horizontalDirection == 0)
            {
                angle = verticalDirection > 0 ? 90 : 270;
            }
            else
            {
                if (verticalDirection > 0)
                {
                    angle = horizontalDirection > 0 ? 45 : 135;
                }
                else
                {
                    angle = horizontalDirection > 0 ? 315 : 225;
                }
            }

            movement.movementDirection = new Vector2(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad));
        }
    }
}
