using UnityEngine;
using System.Collections;

public class SphericController : MonoBehaviour
{

    public float radius = 0.6f;
    public float translateSpeed = 180.0f;
    public float rotateSpeed = 360.0f;
    public float gravity = 1f;
    public float minVSpeed = -15f;

    private bool landed = false;
    private float effectiveRadius;
    private float vSpeed = 0;

    float angle = 0.0f;
    Vector3 direction = Vector3.one;
    Quaternion rotation = Quaternion.identity;
    bool moving = false;
    Animator animator;

    void Start()
    {
        effectiveRadius = 0;
        animator = gameObject.GetComponent<Animator>();
    }

    void Update()
    {
        var translateSpeedY = Input.GetKey(KeyCode.W) ? 5 : (Input.GetKey(KeyCode.S) ? -5 : 0);
        var translateSpeedX = Input.GetKey(KeyCode.A) ? 5 : (Input.GetKey(KeyCode.D) ? -5 : 0);
        //if (Input.GetKey(KeyCode.A)) Translate(translateSpeed, 0);
        //if (Input.GetKey(KeyCode.D)) Translate(-translateSpeed, 0);

        if (translateSpeedX == 0 && translateSpeedY == 0 && moving)
        {
            moving = false;
            animator.SetBool("Moving", moving);
        }
        else if ((translateSpeedX > 0 || translateSpeedY > 0) && !moving)
        {
            moving = true;
            animator.SetBool("Moving", moving);
        }

        direction = new Vector3(Mathf.Sin(angle), Mathf.Cos(angle));
        // Rotate with left/right arrows
        if (Input.GetKey(KeyCode.LeftArrow)) Rotate(rotateSpeed);
        if (Input.GetKey(KeyCode.RightArrow)) Rotate(-rotateSpeed);
        if (Input.GetKey(KeyCode.Space) && landed) vSpeed = 20;

        Translate(translateSpeedX, translateSpeedY);

        // Translate forward/backward with up/down arrows
        //if (Input.GetKey(KeyCode.UpArrow)) Translate(0, translateSpeed);
        //if (Input.GetKey(KeyCode.DownArrow)) Translate(0, -translateSpeed);

        // Translate left/right with A/D. Bad keys but quick test.


        vSpeed = Mathf.Max(minVSpeed, vSpeed - gravity * Time.deltaTime);
        RaycastHit hit;
        Vector3 maxDist = rotation * Vector3.forward * radius;
        if (Physics.Raycast(maxDist, -maxDist, out hit))
        {
            var expectedRadius = radius - hit.distance + 0.8f;
            effectiveRadius += vSpeed * Time.deltaTime;
            if (expectedRadius >= effectiveRadius)
            {
                effectiveRadius = expectedRadius;
                landed = true;
            }
            else
                landed = false;
        }

        UpdatePositionRotation();
    }

    void Rotate(float amount)
    {
        angle += amount * Mathf.Deg2Rad * Time.deltaTime;
    }

    void Translate(float x, float y)
    {
        var perpendicular = Vector3.forward;// new Vector3(-direction.y, direction.x);
        var verticalRotation = Quaternion.AngleAxis(y * Time.deltaTime, perpendicular);
        var horizontalRotation = Quaternion.AngleAxis(x * Time.deltaTime, direction);
        rotation *= horizontalRotation * verticalRotation;
    }

    void UpdatePositionRotation()
    {
        transform.position = rotation * Vector3.forward * effectiveRadius;
        transform.rotation = rotation * Quaternion.LookRotation(direction, Vector3.forward);
    }

    void OnDrawGizmos()
    {
        Vector3 maxDist = rotation * Vector3.forward * radius;
        Vector3 center = Vector3.zero;
        Gizmos.DrawRay(maxDist, -maxDist);
    }
}
